package lightcontainer.domains;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;

public class Server {
    // Variables
    private ServerSocket server;
    private final int server_port;
    private boolean server_run;

    // Constructor
    public Server(int port) {
        this.server_port = port;
        this.server_run = false;
    }

    /**
     * Initializes the server and starts it on the previously selected port.
     *
     * @since       1.0
     *
     * @see         Thread#start()
     * @see         ClientHandler
     */
    public void start() {
        try {
            this.server = new ServerSocket(this.server_port);
            while (server_run) {
                // Accepting connection requests (blocking)
                Socket client = this.server.accept();
                // Create a new Handler client by passing these dependencies to it
                ClientHandler clientHandler = new ClientHandler(client, null); // TODO passer FileFrontEnd ou faire ca dans le repository ?!
                // Add the client handler to its repository (clienthandlerrepository)
                // this.repository.add(clientHandler); TODO REPOSITORY
                // Start the thread
                (new Thread(clientHandler)).start();
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * Stops the server and terminates the new connection.
     *
     * @since       1.0
     */
    public void stop() {
        if (this.server_run) {
            try {
                this.server_run = false;
                this.server.close();
            } catch (IOException ignored) { }
        }
    }
}
