package lightcontainer.domains;

import java.net.DatagramPacket;
import java.net.InetAddress;
import java.net.MulticastSocket;

/**
 * StoreMulticastRunnable
 *
 * Class listening to the announcement of new StoreBackEnd.
 * Allowing it to be used as a storage unit.
 *
 * @version     1.0
 * @since       1.0
 *
 * @see         Runnable
 * @author      Jérémi NIHART <j.nihart@student.helmo.be>
 */
public class StoreMulticastRunnable implements Runnable {
    // Variable
    private final String multicast_address;
    private final int multicast_port;

    private final byte[] buffer = new byte[256];
    private MulticastSocket listener;

    // Constructor
    public StoreMulticastRunnable(String multicast_address, int multicast_port) {
        this.multicast_address = multicast_address;
        this.multicast_port = multicast_port;
    }

    /**
     * Start Multicast listening on indicated port and IP group.
     *
     * @since       1.0
     *
     * @see         MulticastSocket#receive(DatagramPacket) 
     * @see         DatagramPacket
     */
    @Override
    public void run() {
        try {
            // Create a new listening socket
            this.listener = new MulticastSocket(this.multicast_port);
            // Create an identifier for the multicast group on the specified ip
            InetAddress listener_group = InetAddress.getByName(this.multicast_address);
            // Creation of a packet for the information received
            DatagramPacket packet = new DatagramPacket(buffer, buffer.length);
            // Add the listener to the multicast group
            listener.joinGroup(listener_group);
            while(true) {
                listener.receive(packet);
                String data = new String(packet.getData(), 0, packet.getLength());

                System.out.println(data); // TODO ajouter un controller et lui signaler qu'il y a un nouveau StoreBackEnd
            }
        } catch (Exception ignore) { }
    }

    /**
     * Closes the MulticastSocket connection and aborts the listening and infinite listening loop.
     *
     * @since       1.0
     *
     * @see         StoreMulticastRunnable#run()
     */
    public void stop() {
        this.listener.close();
    }
}
