package lightcontainer.domains;

import lightcontainer.interfaces.StoreProcessorFFE;

import java.io.*;
import java.net.Socket;
import java.nio.charset.StandardCharsets;

/**
 * StoreProcessor
 *
 * Class communicating with the storebackend and
 * processing the tasks in the FileFrontEnd
 *
 * @version     1.0
 * @since       1.0
 *
 * @see         Runnable
 * @see         AutoCloseable
 * @author      Jérémi NIHART <j.nihart@student.helmo.be>
 */
public class StoreProcessor implements Runnable, AutoCloseable {
    // Variables
    private final StoreProcessorFFE fileFrontEnd;
    private final Socket store;

    private BufferedReader reader;
    private PrintWriter writer;

    // Constructor
    public StoreProcessor(Socket socket, StoreProcessorFFE ffe) {
        this.fileFrontEnd = ffe;
        this.store = socket;
        initStore();
    }

    /**
     * Initialise the Store's Reader and Writer.
     *
     * @since       1.0
     *
     * @see         BufferedReader
     * @see         PrintWriter
     */
    private void initStore() {
        try {
            this.reader = new BufferedReader(new InputStreamReader(
                    this.store.getInputStream(),
                    StandardCharsets.UTF_8
            ));
            this.writer = new PrintWriter(new OutputStreamWriter(
                    this.store.getOutputStream(),
                    StandardCharsets.UTF_8
            ), true);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * Thread Function
     * Start the dialogue with the storebackend.
     *
     * @since       1.0
     */
    @Override
    public void run() {
        while (true) {
            try {
                // Envoie donnée au StoreBackEnd
                String command = this.reader.readLine();
                // TODO gestion de la réception de commandes, fichier, ...
                if (command != null) System.out.println("StoreBackEnd: " + command);

                // Réception donnée du StoreBackEnd
            } catch (IOException e) {
                e.printStackTrace();
            }

            // Alerter le FileFrontEnd que ce store processor est disponible pour gérer de nouvelle demande
            fileFrontEnd.onStoreAvailable(this);
        }
    }

    /**
     * AutoClosable Function
     * Close the Storage thread and resources.
     *
     * @since       1.0
     */
    @Override
    public void close() {

    }
}
